<?php
/**
* API to generate google maps with marker
* for google Map API V2
* @package     jgmap
* @subpackage  
* @author      Emmanuel HESRY
* @contributor
* @copyright   2009 Emmanuel HESRY
* @link        http://www.jelix.org
* @licence     GNU Lesser General Public Licence see LICENCE file or http://www.gnu.org/licenses/lgpl.html
*
*  This class was strongly inspired from GoogleMapAPI (http://www.phpinsider.com/php/code/GoogleMapAPI/ - copyright 2005 - LGPL licence)
*  This class was rewritten/adapted/improved for Jelix by Emmanuel HESRY
*
*/

/**
 * jGmapPoint
 * @package  jgmap
 */
class jGmapPoint {
    public $lat = 0;
    public $lng = 0;

    private function __construct($lat, $lng) {
        if (is_numeric($lat) and is_numeric($lng)) {
            $this->lat = $lat;
            $this->lng = $lng;
        }
    }
    public static function createPoint($lat, $lng) {
        return new jGmapPoint($lat,$lng);
    }
}

/**
 * jGmapIcon
 * @package  jgmap
 */
class jGmapIcon {
    const DEFAULT_ICON = "google.maps.DEFAULT_ICON";
    
    /* icon's id to find it in list of icons */
    public $id = ''; /* md5 key from icon image's name to find it in list of icons */
    public $params = array (
        'image' => '', /* image's file name */
        'shadow' => '', /* shadow image's file name */
        'width' => 0, /* width and heigth of icon's image */
        'height' => 0,
        'shadowWidth' => 0, /* width and heigth of icon's image */
        'shadowHeight' => 0,
        'anchorX' => 0 , /* icon's coordinates */
        'anchorY' => 0 ,
        'infoWindowAnchorX' => 0 , /* infoWindow's coordinates */
        'infoWindowAnchorY' => 0
    );
	/**
	* instanciate an icon object
	* @param $params an array of parameters to define icon (image, shadow, anchorX, anchorY, infoWindowAnchorX, infoWindowAnchorY)
	* all parameters are optionnal except image 
	*/
    private function __construct($params) { 
        $iconImagePath = preg_match('#^https?://#',$params['image']) ? $params['image'] : $_SERVER['DOCUMENT_ROOT'].$params['image'];
        if(!($imageInfo = @getimagesize($iconImagePath))) {
            throw new jException('jgmap~gmap.image.not.exist',array($iconImagePath));   
        }
        /* shadow is optionnal parameter */
        if (array_key_exists('shadow',$params)) {
            $shadowImagePath = preg_match('#^https?://#',$params['shadow']) ? $params['shadow'] : $_SERVER['DOCUMENT_ROOT'].$params['shadow'];
            if(!($shadowInfo = @getimagesize($shadowImagePath))) {
                throw new jException('jgmap~gmap.image.not.exist',array($shadowImagePath));   
            }
            $this->params['shadowWidth'] = $shadowInfo[0];
            $this->params['shadowHeight'] = $shadowInfo[1];
        }
        foreach ($params as $key => $value) {
            $this->params[$key] = $value;
        }
        $this->id = md5($this->params['image']);
        $this->params['width'] = $imageInfo[0];
        $this->params['height'] = $imageInfo[1];
        if($this->params['anchorX'] == 0) {
            $this->params['anchorX'] = $imageInfo[0] / 2;
        }
        if($this->params['anchorY'] == 0) {
            $this->params['anchorY'] = $imageInfo[1] / 2;
        }
        if($this->params['infoWindowAnchorX'] == 0) {
            $this->params['infoWindowAnchorX'] = $imageInfo[0] / 2;
        }
        if($this->params['infoWindowAnchorY'] == 0) {
            $this->params['infoWindowAnchorY'] = $imageInfo[1] / 2;
        }                     
    }
    /**
	* create icon object
	* @see __construct()
	*/
    public static function createIcon($params = array()) {
        if (!empty($params)) {
            return new jGmapIcon($params);
        } else {
            return jGmapIcon::DEFAULT_ICON;
        }
    }
}
/**
 * jGmapMarker
 * @package  jgmap
 */
class jGmapMarker {
    public $point = null;
    public $markerOptions = array (
        'draggable' => false,
        'clickable' => false,
        'title' => 'Jelix',
        'icon' => jGmapIcon::DEFAULT_ICON
    );
    /**
	* instanciate a marker
	* @param $lat latittude coordinate
	* @param $lng longitude coordinate
	* @param $options a array of options (title, draggable, clickable, icon)
	*/
	private function __construct($lat, $lng, $options) {
        global $configJgmap;
	    $this->point = jGmapPoint::createPoint($lat, $lng);
        /* default options for all marker from configuration file */
    	if (!empty($configJgmap['markerOptions'])) {
    	    $this->markerOptions = $configJgmap['markerOptions'];
    	}
        /* if options not empty change default marker options by this one */
    	if (!empty($options)) {
            foreach ($options as $option => $value) {
                $this->markerOptions[$option] = $value;
            }
    	}
	}
    /**
	* create a marker object
	* @see __construct()
	*/
    public static function createMarker($lat, $lng, $options=array()) {
        return new JGmapMarker($lat, $lng, $options);
    }
}
/**
 * jGmap
 * @package  jgmap
 */
abstract class jGmap {
	const NORMAL_MAP = "NORMAL_MAP";
	const SATELLITE_MAP = "SATELLITE_MAP";
	const HYBRID_MAP = "HYBRID_MAP";
	const PHYSICAL_MAP = "PHYSICAL_MAP";
	const DRAGGING = "Dragging";
	const INFO_WINDOW = "InfoWindow";
	const DOUBLE_CLICK_ZOOM = "DoubleClickZoom";
	const CONTINUOUS_ZOOM = "ContinuousZoom";
	const SCROLL_WHEEL_ZOOM = "ScrollWheelZoom";
	const PINCH_TO_ZOOM = "PinchToZoom";
	const DEFAULT_VALUE = 0;
	const MENU = 1;
	const SMALL = 0;
	const LARGE = 1;
	const SMALLZOOM = 2;
	const LARGE3D = 3;
	const SMALLZOOM3D = 4;
    const DEFAULT_ZOOM = 16;
    const MAX_COORDINATE = -1000000;
    const MIN_COORDINATE = 1000000;
	/* default version for google map API */
	public $googleMapApiVersion = "V2";
	/* but for external geocoding service */
	public $googleKey = ""; 
	/* name's map in page*/
	public $mapDomId = "_jGmap";
	/* session variable name */
	private $mapVarSession = "jGmap";
	/* width map'container in page */
	public $mapCssWidth = "380px";
	/* heigth map's container in page */
	public $mapCssHeight = "320px";
	/* color behind the map's tiles and appears where we are zooming for example (default: E5E3DF) */ 
	public $mapBackgroundColor = "#E5E3DF";
	public $mapZoom = self::DEFAULT_ZOOM;
	/* array of map's option to enable or disable features */
	public $mapOptions = array (
		self::DRAGGING => true,
		self::INFO_WINDOW => true,
		self::DOUBLE_CLICK_ZOOM => true,
		self::CONTINUOUS_ZOOM => false, // smoth animation when zoomin the map
		self::SCROLL_WHEEL_ZOOM => true,
		self::PINCH_TO_ZOOM => true // specific zoomin for iPhone or iPod touch screen
	);
	/* enable/disable map control */
	public $mapControl = true;
	/* array of UI options to customize default map's UI */
	public $mapControlType = self::LARGE3D;
	/* enable/disable mapType control */
	public $mapTypeControl = true;
	/* selected default map type */
	public $mapType = self::NORMAL_MAP;
	/* mapType Control style DEFAULT_VALUE or MENU*/
	public $mapTypeControlStyle = self::MENU;
	/* array of mapType enable in mapType Control */
	public $mapTypes = array(
		self::NORMAL_MAP => true,
		self::SATELLITE_MAP => true,
		self::HYBRID_MAP => true,
		self::PHYSICAL_MAP => false
	);
	/* enable/disable scale control */
	public $scaleControl = false;
	/* enable/disable scale control */
	public $overviewMapControl = false;
	/* enable/disable keyboard handler */
	public $keyboardHandler = true;
    /* all markers in map */
    public $markers = array();
    /* all icons in map */
    public $icons = array();
    /* for placing map's center and adjust zoom level */
    public $maxPoint = null;
    public $minPoint = null;
    public $centerPoint = null;
    public $adjustingZoomFactor = 0.01;
	function __construct($mapDomId="_jGmap") {
	    global $configJgmap;
	    /* read configuration file */
		$configJgmap = parse_ini_file(JELIX_APP_VAR_PATH.'config/jgmap.module.ini.php', true);
		if (!empty($configJgmap)) {
    		foreach ($configJgmap['generalOptions'] as $key => $value) {
    		    if ($key == "mapDomId") {
    		        if ($value == "") {
        		        $this->$key = $mapDomId."_".str_replace(array(' ', '.'), array('', ''), microtime());
    		        } else {
        		        $this->$key = $value."_".str_replace(array(' ', '.'), array('', ''), microtime());
    		        }
    		    } elseif ($key == "mapVarSession") {
    		        if ($value != "") {
        		       $this->$key = $value;
    		        }
    		    } else {
            		$this->$key = $value;
        		}
        	}
        	if (!empty($configJgmap['mapOptions'])) {
        	    $this->mapOptions = $configJgmap['mapOptions'];
        	}
    		foreach ($configJgmap['mapControl'] as $key => $value) {
        		$this->$key = $value;
        	}
    		foreach ($configJgmap['mapTypeControl'] as $key => $value) {
        		$this->$key = $value;
        	}
        	if (!empty($configJgmap['mapTypes'])) {
        	    $this->mapTypes = $configJgmap['mapTypes'];
        	}
    		foreach ($configJgmap['otherUIControl'] as $key => $value) {
        		$this->$key = $value;
        	}
        }
        /* initialize maxPoint, minPoint and centerPoint */
        $this->maxPoint = jGmapPoint::createPoint(self::MAX_COORDINATE,self::MAX_COORDINATE);
        $this->minPoint = jGmapPoint::createPoint(self::MIN_COORDINATE,self::MIN_COORDINATE);
        $this->centerPoint = jGmapPoint::createPoint(0,0);
        
        /* put in session a call to google map api starts */
        if (!isset($_SESSION[$this->mapVarSession])) $_SESSION[$this->mapVarSession] = false;
	}
	
    /**
	* add a marker icon's in list of icons
	* @param array $params
	* @return string icon id
	*/
	function addMarkerIcon($params=array()) {
        if (!empty($params) and array_key_exists('image',$params) and $params['image']!='') {
            $md5Key =  md5($params['image']);
            /* search in list of icons with md5 key */
            /* return icon object which already exist in list */
            if (array_key_exists($md5Key, $this->icons) and $this->icons[$md5Key] instanceof jGmapIcon) {
                $icon = $this->icons[$md5Key];
            } else {
            /* else create a new icon object */
            $icon = jGmapIcon::createIcon($params);
            $this->icons[$icon->id] = $icon;
            }
            return $icon->id;
        } else {
            throw new jException("jgmap~gmap.icon.empty.parameters");
        }
	}
    /**
	* add a marker in list of markers
	* @param float $lat
	* @param float $lng
	* @param array $options
	*/
	function addMarkerByCoords($lat, $lng, $options=array()) {
	    $marker = jGmapMarker::createMarker($lat, $lng, $options);
	    $this->markers[] = $marker;
        $this->setCenterMaxMinPoint($marker);
	}
    /**
	* set max and min latittude, longitude and map's center after adding a marker
	* @param jGmapMarker $marker
	*/
	private function setCenterMaxMinPoint($marker) {
        $this->maxPoint->lat = max($marker->point->lat, $this->maxPoint->lat);
        $this->maxPoint->lng = max($marker->point->lng, $this->maxPoint->lng);
        $this->minPoint->lat = min($marker->point->lat, $this->minPoint->lat);
        $this->minPoint->lng = min($marker->point->lng, $this->minPoint->lng);
        $this->centerPoint->lat = ($this->minPoint->lat + $this->maxPoint->lat) / 2;        
        $this->centerPoint->lng = ($this->minPoint->lng + $this->maxPoint->lng) / 2;
    }
    /**
	* adjust max and min points to be sure all markers will be in map
	* to do only when all markers added
	* @param jGmapMarker $marker
	*/
	private function adjustMaxMinPoint() {
        /* length between max and min latittude */
        $lenLat = ($this->maxPoint->lat - $this->minPoint->lat);
        /* length between max and min longitude */
        $lenLng = ($this->maxPoint->lng - $this->minPoint->lng);
        $this->maxPoint->lat = $this->maxPoint->lat + $lenLat*$this->adjustingZoomFactor;
        $this->maxPoint->lng = $this->maxPoint->lng + $lenLng*$this->adjustingZoomFactor;
        $this->minPoint->lat = $this->minPoint->lat - $lenLat*$this->adjustingZoomFactor;
        $this->minPoint->lng = $this->minPoint->lng - $lenLng*$this->adjustingZoomFactor;
    }
    /**
	* return google map API script loading
	* @return string
	*/
    static function getHeaderJS($googleKey) {
		$headerJS = new jTpl();
		$headerJS->assign('googleKey',$googleKey);
        return $headerJS->Fetch("jgmap~headerGoogleMap");
    }
	/** 
	 * return map script initialization 
	* @return string
	 */
	function getInitMapJS() {
        $this->adjustMaxMinPoint();
		$initMapJS = new jTpl();
		$initMapJS->assign('map',$this);
        return $initMapJS->Fetch("jgmap~initMap");
	}

	/** 
	 * return map script common functions
	* @return string
	 */
	function getCommonJS() {
        $this->adjustMaxMinPoint();
		$commonJS = new jTpl();
		$commonJS->assign('map',$this);
        return $commonJS->Fetch("jgmap~commonFunction");
	}
	/** 
	 * return map variable session name
	* @return string
	 */
	function getMapVarSession() {
        return $this->mapVarSession;
	}
	/** 
	 * return map variable session name
	* @return string
	 */
	static function terminate($map) {
        unset($_SESSION[$map->getMapVarSession()]);
	}
	function getSmallMapControl() {
		return self::SMALL;
	}
	function getLargeMapControl() {
		return self::LARGE;
	}
	function getSmallZoomControl() {
		return self::SMALLZOOM;
	}
	function getLarge3DMapControl() {
		return self::LARGE3D;
	}
	function getSmallZoom3DControl() {
		return self::SMALLZOOM3D;
	}
}
?>
